function alignedShapes = alignShapes(allShapes, scaling)

% Pre-allocate
n_shapes = size(allShapes,2);
alignedShapes = zeros(size(allShapes));

% Mean shape across all subjects (assuming each shape in totalShapes is a different subj)
meanShape = mean(allShapes,2); % x1, y1, x2, y2, ..., x20, y20
meanShape = [meanShape(1:2:end) meanShape(2:2:end)]; % Reshape for Procrustes

%% Loop thru each shape in totalShapes and transform via Procrustes analysis
for n_shape = 1:n_shapes
    % Landmarks shape for the current subject (if 1 shape per subject)    
    iShape = [allShapes(1:2:end,n_shape) allShapes(2:2:end,n_shape)];
    
    % Do the Procrustes alignment
    [~, iShapeAligned] = procrustes(meanShape,iShape,'scaling',scaling,'reflection','best');
    
    % Store the aligned shape in a similar manner as how totalShapes is passed in            
    alignedShapes(1:2:end,n_shape) = iShapeAligned(:,1)';
    alignedShapes(2:2:end,n_shape) = iShapeAligned(:,2)';            
end

end % End of main