
project_dir = pwd;
addpath(project_dir,fullfile(project_dir,'Utilities'),fullfile(project_dir,'Visualization')), cd(project_dir)

% Load landmarks and images
pathToLandmarks = '../../Data/landmarks/';
pathToImages = '../../Data/images/';

% Load landmarks    
C = importdata(fullfile(pathToLandmarks,'fish60_fin.csv'));
landmarks = C.data(1:end,2:end);
faceFiles = char(C.textdata(2:end,1));

allLandmarks = landmarks';
im = load(fullfile(pathToImages,faceFiles(1,:)));
allLandmarks(1:2:end,:) =    allLandmarks(1:2:end,:) + size(im.Intensity,2)/2; % Adjust for coordinate differences
allLandmarks(2:2:end,:) = -1*allLandmarks(2:2:end,:) + size(im.Intensity,1)/2;   
% save('Example_FindFace_landmarks_MUCT','allLandmarks')

%% View an image with landmarks
n_im = 1;
im = load(fullfile(pathToImages,faceFiles(n_im,:)));

figure, imshow(im.Intensity,[]), hold on
plot(allLandmarks(1:2:end,n_im),allLandmarks(2:2:end,n_im),'-ro','MarkerSize', 2), hold off

%% Create the shape model from the unaligned shapes
shapeModel = buildShapeModel(allLandmarks,faceFiles);
plotLandmarks(shapeModel.alignedShapes,'layout','fish');

save('../../Data/FishShapeModel.mat','shapeModel');

%% Explore the shape model
guiPrinComps(shapeModel,'layout','fish'); % Effect of PC weights on shape (GUI)
plotPrinComp(shapeModel,3,'fish'); % Plot variations of the different PCs

%% Create the gray-level 2D profile model
grayModel = buildGrayLevelModel(pathToImages,shapeModel,'resolutions',[4 2 1],'region_size',[20 10]);

save('../../Data/FishGrayModel.mat','grayModel');

%% Explore the gray model
n_resolution = 3;
n_landmark = 1;
viewGrayProfile(grayModel,n_resolution,n_landmark)

%% Find a face!
%findFace(im_to_segment,InitPoint,im_name,shapeModel,grayModel,'layout','fish','evolutions',5)
